#include <cstdlib>
#include <exception>

class Option{
protected:
  const char *const name;
public:
  int checkname(const char* nm) const;
  Option(const char* nm):name(nm){};
  virtual void setoption(const char *val)=0;
  virtual const void* getoptionval()=0;
  //calling function should know what type of value is returned.
  virtual void apply(){}; //for setup application of option.
  virtual void deferedapply(){}; //for deferred application of option.
  virtual unsigned int process(const char *name,const char *value){return 0;};
  virtual void close(){};
  //Allows options to receive updates in subsequent options.
};

class TrueFalseOption: public Option{
protected:
  unsigned int value;
public:
  TrueFalseOption(const char* nm):Option(nm){};
  virtual void setoption(const char *val);
  virtual const void* getoptionval() {return &value;};
  virtual void apply(){};
};


class IntOption: public Option{
protected:
  unsigned int value;
public:
 IntOption(const char* nm):Option(nm),value(0){};
  virtual void setoption(const char *val);
  virtual const void* getoptionval() {return &value;};
  virtual void apply(){};
};

class ModeOption: public IntOption{
  //First mode is default value.
protected:
  const char *const *modes;
public:
 ModeOption(const char* nm,const char *const *mds):IntOption(nm),modes(mds){};
  virtual void setoption(const char *val);
};

class SetSeedOption: public IntOption{
public:
  SetSeedOption(const char* nm):IntOption(nm){};
  void apply();
};

class PointerOption: public Option{
protected:
  const void *data;//create some structure and save a pointer to it.
  const char *value;
public:
 PointerOption(const char* nm):Option(nm),data(NULL),value(NULL){};
  void setoption(const char *val);
  const void* getoptionval() {return data;};
  //  virtual void apply();
};

class NPestimatorOption:public PointerOption{
public:
 NPestimatorOption(const char *nm):PointerOption(nm){};
  virtual void apply();
};

class PairNPestimatorOption:public PointerOption{
  //Special formatting, may need to do something different
public:
 PairNPestimatorOption(const char *nm):PointerOption(nm){};
  virtual void apply();
};

class OutputFileStreamOption:public PointerOption{
public:
 OutputFileStreamOption(const char *nm):PointerOption(nm){};
  virtual void apply();
  virtual void close();  
};

class InputFileStreamOption:public PointerOption{
public:
 InputFileStreamOption(const char *nm):PointerOption(nm){};
  virtual void apply();
  virtual void close();  
};


class DistributionOption:public PointerOption{
  unsigned int maxpars;
  unsigned int npset;
 public:
 DistributionOption(const char *nm):PointerOption(nm),maxpars(0),npset(0){};
  virtual void apply();
  virtual unsigned int process(const char *na,const char *val);
};

class DelayedProcessOption:public PointerOption{
  //saves a list of arguments then processes them lazily to create something.
 protected:
  unsigned int numparameters;
  static const unsigned int maxallowed_parameters=10;
  const char *const*allowedparameters;
  const char **parameters;
  const char **values;
 public:
 DelayedProcessOption(const char *nm,const char *const *allpar):PointerOption(nm),numparameters(0),allowedparameters(allpar){parameters=new const char *[maxallowed_parameters];values=new const char *[maxallowed_parameters];};
  virtual const void* getoptionval()=0;
  virtual unsigned int process(const char *na,const char *val);
};





class DistortionOption:public DelayedProcessOption{
 public:
 DistortionOption(const char *nm,const char *const *allpar):DelayedProcessOption(nm,allpar){};
  virtual const void* getoptionval();
};

class AdequacyTestOption:public PointerOption{
 public:
 AdequacyTestOption(const char *nm):PointerOption(nm){};
  virtual void apply();
};



class option_formatting_error: public std::exception{
  static const char*const header;
  char * message;
public:
  option_formatting_error(const char *const mess);
  option_formatting_error(const char *const mess,const char*values,const char *end);
  virtual const char* what() const throw();
};

